#include <cstdlib>
#include <iostream>

#define YESNO(option) ((option)? "yes": "no")

inline void check_cuda_error(const char* cudaop, const char* source,
       unsigned int line, cudaError_t error) {
   if (error != cudaSuccess) {
      std::cerr << cudaop << " at " << source << ":" << line
	 << " failed: " << cudaGetErrorString(error) << std::endl;
      exit(1);
   }
}

#define CHECK_CUDA(opname, ...) \
   check_cuda_error(#opname, __FILE__, __LINE__, opname(__VA_ARGS__))

int main() {
   int device; CHECK_CUDA(cudaGetDevice, &device);
   int device_count; CHECK_CUDA(cudaGetDeviceCount, &device_count);
   struct cudaDeviceProp device_prop;
   CHECK_CUDA(cudaGetDeviceProperties, &device_prop, device);
   if (device_count > 1) {
      std::cout << "device " << device << " selected out of "
	 << device_count << " devices:" << std::endl;
   } else {
      std::cout << "one device present:" << std::endl;
   }
   std::cout << "name: " << device_prop.name << std::endl;
   std::cout << "compute capability: " << device_prop.major
      << "." << device_prop.minor << std::endl;
   std::cout << "total global memory: " << device_prop.totalGlobalMem
      << std::endl;
   std::cout << "total constant memory: " << device_prop.totalConstMem
      << std::endl;
   std::cout << "total shared memory per block: "
      << device_prop.sharedMemPerBlock << std::endl;
   std::cout << "registers per block: " << device_prop.regsPerBlock
      << std::endl;
   std::cout << "warp size: " << device_prop.warpSize << std::endl;
   std::cout << "mem pitch: " << device_prop.memPitch << std::endl;
   std::cout << "max threads per block: " << device_prop.maxThreadsPerBlock
      << std::endl;;
   std::cout << "max threads dim: " << device_prop.maxThreadsDim[0] << " "
      << device_prop.maxThreadsDim[1] << " " << device_prop.maxThreadsDim[2]
      << std::endl;
   std::cout << "max grid dim: " << device_prop.maxGridSize[0] << " "
      << device_prop.maxGridSize[1] << " " << device_prop.maxGridSize[2]
      << std::endl;
   std::cout << "multi processor count: " << device_prop.multiProcessorCount
      << std::endl;
   std::cout << "kernel exec timeout enabled: " <<
      YESNO(device_prop.kernelExecTimeoutEnabled) << std::endl;
   std::cout << "device overlap: " << YESNO(device_prop.deviceOverlap)
      << std::endl;
   std::cout << "integrated: " << YESNO(device_prop.integrated) << std::endl;
   std::cout << "can map host memory: "
      << YESNO(device_prop.canMapHostMemory) << std::endl;
   std::cout << "unified addressing: " << YESNO(device_prop.unifiedAddressing)
      << std::endl;
}
